@@ -8,6 +8,10 @@
 #------------------------------------------------------------------------------
 # Miscellaneous utilities
 
+stop0 <- function(...) stop(..., call.=FALSE)
+
+warning0 <- function(...) warning(..., call.=FALSE)
+
 any1 <- function(x) any(x != 0) # like any but no warning if x not logical
 
 # The function stopif is intended for catching programmer errors.
@@ -15,10 +19,6 @@ any1 <- function(x) any(x != 0) # like any but no warning if x not logical
 
 stopif <- function(...) stopifnot(!(...))
 
-stop0 <- function(...) stop(..., call.=FALSE)
-
-warning0 <- function(...) warning(..., call.=FALSE)
-
 cat0 <- function(...) cat(..., sep="")      # cat with no added spaces
 
 paste0 <- function(...) paste(..., sep="")  # paste with no added spaces
@@ -37,8 +37,6 @@ pastef <- function(s, format, ...) # paste the c printf style args to s
 
 strip.white.space <- function(s) gsub("[ \t\n]", "", s)
 
-is.try.error <- function(obj) class(obj)[1] == "try-error"
-
 # is.na.or.zero's main purpose is to see if a plot component should be
 # drawn, i.e., to see if the component "has a color"
 # We use identical() and not is.na() below because is.na(x) gives warnings
@@ -47,33 +45,28 @@ is.try.error <- function(obj) class(obj)[1] == "try-error"
 
 is.na.or.zero <- function(x) identical(x, NA) || identical(x, 0)
 
-warn.if.dots.used <- function(func.name, ...)
+is.try.error <- function(obj) class(obj)[1] == "try-error"
+
+warn.if.dots.used <- function(func.name, ..., stop=FALSE)
 {
     dots <- match.call(expand.dots = FALSE)$...
-    if(length(dots) == 1)
-        warning0(func.name, " ignored unrecognized argument \"", names(dots), "\"")
-    else if(length(dots) > 1)
-        warning0(func.name, " ignored unrecognized arguments ",
-                 paste.quoted.names(names(dots)))
-}
-
-stopifnot.boolean <- function(b) # b==0 or b==1 is also ok
-{
-    if(length(b) != 1 || !(is.logical(b) || is.numeric(b)) ||
-            is.na(b) || !(b == 0 || b == 1)) {
-        name <- deparse(substitute(b))
-        cat0("\n", name, ": ")
-        print(b)
-        cat("\n")
-        stop0("the ", name,
-              " argument is not FALSE or TRUE or 0 or 1 (see above print)")
+    if(length(dots)) {
+        if(stop)
+            stop0(func.name, ": unrecognized argument \"", names(dots)[1], "\"")
+        else
+            warning0(func.name, " ignored unrecognized argument \"", names(dots)[1], "\"")
     }
 }
+stop.if.dots.used <- function(func.name, ...)
+{
+    warn.if.dots.used(func.name, ..., stop=TRUE)
+}
+
 check.classname <- function(object, object.name, class.names)
 {
     if(!inherits(object, class.names))
         stop0("the class of \"", object.name, "\" is not \"",
-            paste(class.names, collapse="\" or \""), "\"")
+              paste(class.names, collapse="\" or \""), "\"")
 }
 
 to.logical <- function(x, len)
@@ -102,29 +95,6 @@ make.call.generic <- function(Call, func.name)
     Call
 }
 
-# warn.if.not.all.finite is intended to help clarify possibly confusing
-# messages from within the bowels of calls to other functions later
-# Return TRUE if warning issued.
-
-warn.if.not.all.finite <- function(x, text="unknown")
-{
-    is.factors <- sapply(x, is.factor)
-    if(any(is.factors)) {
-        if(NCOL(x) == 1 || all(is.factors)) # TODO suspect
-            return(FALSE)
-        x <- x[, !is.factors]               # remove factor columns before is.finite check
-    }
-    if(any(sapply(x, is.na))) {
-        warning0("NA in ", text)
-        return(TRUE)
-    }
-    if(!all(sapply(x, is.finite))) {
-        warning0("non finite value in ", text)
-        return(TRUE)
-    }
-    FALSE
-}
-
 my.fixed.point <- function(x, digits)
 {
     if(NROW(x) > 2) # if only intercept term and one other then don't use fixed point
@@ -141,7 +111,7 @@ check.index.vec <- function(index.name, indexVec, object,
                         check.empty = FALSE, use.as.col.index=FALSE,
                         allow.negative.indices = TRUE,
                         allow.duplicates = FALSE,
-                        allow.zeroes = FALSE)
+                        allow.zeroes = FALSE, treat.NA.as.one=FALSE)
 {
     if(is.null(indexVec)) {
         if(check.empty)
@@ -150,10 +120,12 @@ check.index.vec <- function(index.name, indexVec, object,
     }
     if(any(is.na(indexVec)))
         stop0("NA in \"", index.name, "\"")
+    if(treat.NA.as.one && (is.na(indexVec)[1] && length(indexVec) == 1))
+        indexVec <- 1
     if(!(NROW(indexVec) == 1 || NCOL(indexVec) == 1))
         stop0("\"", index.name, "\" must be a vector not a matrix ",
-            "(\"", index.name, "\" has dimensions ",
-            NROW(indexVec), " x ", NCOL(indexVec), ")")
+              "(\"", index.name, "\" has dimensions ",
+              NROW(indexVec), " x ", NCOL(indexVec), ")")
 
     if(use.as.col.index)
         len <- NCOL(object)         # index is for cols of object
@@ -172,8 +144,8 @@ check.index.vec <- function(index.name, indexVec, object,
         # note that a single FALSE or TRUE is ok regardless of length(object)
         if(length(indexVec) > len && length(indexVec) != 1) {
             stop0("logical index vector \"", index.name, "\" is too long\n",
-                "       Its length is ", length(indexVec),
-                " and the max allowed length is ", len)
+                  "       Its length is ", length(indexVec),
+                  " and the max allowed length is ", len)
         }
     } else if(is.numeric(indexVec)) {
         if(check.empty) {
@@ -185,6 +157,8 @@ check.index.vec <- function(index.name, indexVec, object,
                 else
                     stop0("\"", index.name, "\" is all zeroes")
         }
+        if(any(floor(indexVec) != indexVec))
+            stop0(index.name, " is not an integer")
         if(any(indexVec < 0) && any(indexVec > 0))
             stop0("mixed negative and positive values in \"", index.name, "\"")
         if(!allow.zeroes && any(indexVec == 0) && length(indexVec) != 1)
@@ -193,44 +167,23 @@ check.index.vec <- function(index.name, indexVec, object,
             warning0("duplicates in \"", index.name, "\"")
         if(!allow.negative.indices && any(indexVec < 0))
             stop0("negative value in \"", index.name, "\"")
-        if(any(abs(indexVec) > len))
-            if(len == 1)
+        if(any(abs(indexVec) > len)) {
+            if(len != 1)
                 stop0("out of range value in \"", index.name,
-                    "\" (the only legal value is 1)")
+                      "\" (allowed index range is 1:",  len, ")")
+            else if(treat.NA.as.one)
+                stop0("out of range value in \"", index.name,
+                      "\" (the only legal value is NA or 1)")
             else
                 stop0("out of range value in \"", index.name,
-                    "\" (allowed index range is 1:",  len, ")")
+                      "\" (the only legal value is 1)")
+        }
     } else
         warning0("index vector \"", index.name,
-            "\" has an unusual class \"", class(indexVec)[1], "\"")
+                 "\" has an unusual class \"", class(indexVec)[1], "\"")
     indexVec
 }
 
-exists.and.not.null <- function(object, mode="any", argname="")
-{
-    # following "if" is like is.null(object) but no error msg if object doesn't exist
-
-    if(paste0("'", object, "'") == "'NULL'")
-        return(FALSE)
-
-    if(paste0("'", object, "'") == "'NA'")
-        if(length(argname))
-            stop0(argname, "=NA")
-        else
-            stop0(object, "illegal NA")
-
-    # TODO removed until I can get this to work reliably
-    #
-    #   if(!exists(paste0("'", object, "'"), where=parent.frame(), mode=mode))
-    #       if(length(argname))
-    #           stop0("you specified ", argname, "=", object,
-    #               " but there is no such ", if(mode=="any") "object" else mode)
-    #       else
-    #           stop0(object, ": no such", if(mode=="any") "object" else mode)
-
-    return(TRUE)
-}
-
 # Return a vector of n clearly distinguishable colors.  Here distinguishability
 # comes before aesthetics and "matching colors" or "ordered" sequences.
 # The first three are also distinguishable on (my) monochrome printer.
@@ -283,28 +236,32 @@ my.print.call <- function(msg, Call)
 }
 
 #------------------------------------------------------------------------------
-
 match.arg1 <- function(arg)     # match.arg1 returns an integer
 {
     formal.args <- formals(sys.function(sys.parent()))
     arg.name=deparse(substitute(arg))
-    match.choices(arg, choices=eval(formal.args[[arg.name]]),  arg.name=arg.name)
+    match.choices(arg[1], choices=eval(formal.args[[arg.name]]), arg.name=arg.name)
 }
 
-match.choices <- function(arg, choices, arg.name)   # choices is a vector of strings
+match.choices <- function(arg, choices, arg.name) # choices is a vector of strings
 {
-    stopifnot(is.character(arg))
-    stopif(is.null(choices))    # indicates a programming error
-    if(all(arg == choices))
-        return(1)
-    i <- pmatch(arg, choices)
-    if(any(is.na(i)))
-        stop0(paste0("bad \"", arg.name, "\" argument \"", arg, "\"\n",
-              "Choose one of: ", paste.quoted.names(choices)))
-    if(i == 0)
-        stop0(paste("the \"", arg.name, "\" argument is ambiguous\n",
-              "Choose one of: ", paste.quoted.names(choices)))
-    i
+    if(!is.character(arg) || length(arg) != 1)
+         stop0("bad \"", arg.name, "\" argument.\n",
+               "Choose one of: ", paste.quoted.names(choices))
+    imatch <- pmatch(arg, choices)
+    if(any(is.na(imatch))) {
+        imatch <- NULL
+        for(i in 1:length(choices))
+            if(pmatch(arg, choices[i], nomatch=0))
+                imatch <- c(i, imatch)
+        if(length(imatch) == 0)
+            stop0(arg.name, "=\"", arg, "\" is illegal.\n",
+                  "Choose one of: ", paste.quoted.names(choices))
+        if(length(imatch) > 1)
+            stop0(paste0(arg.name, "=\"", arg, "\" is ambiguous.\n",
+                         "Choose one of: ", paste.quoted.names(choices)))
+    }
+    imatch
 }
 
 #------------------------------------------------------------------------------
@@ -348,27 +305,6 @@ make.space.for.caption <- function(caption)
         oma[3] <- 3
         par(oma=oma)
     }
-    NULL
-}
-
-make.space.for.bottom.axis <- function()
-{
-    mar <- par("mar")
-    if(mar[1] < 3) {
-        mar[1] <- 3
-        par(mar=mar)
-    }
-    NULL
-}
-
-make.space.for.left.axis <- function()
-{
-    mar <- par("mar")
-    if(mar[2] < 3) {
-        mar[2] <- 3
-        par(mar=mar)
-    }
-    NULL
 }
 
 make.space.for.right.axis <- function()
@@ -378,7 +314,6 @@ make.space.for.right.axis <- function()
         mar[4] <- 3.5
         par(mar=mar)
     }
-    NULL
 }
 
 #------------------------------------------------------------------------------
